<?php

namespace Drupal\og_default_image\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\FileUsage\FileUsageInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class OgDefaultImageForm.
 *
 * Provides a form for setting a default OG image and selecting an image style.
 */
class OgDefaultImageForm extends ConfigFormBase {

  /**
   * @var
   */
  protected $entityTypeManager;

  /**
   * @var
   */
  protected $fileUsage;

  /**
   * Class constructor.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, FileUsageInterface $file_usage) {
    $this->entityTypeManager = $entity_type_manager;
    $this->fileUsage = $file_usage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('file.usage')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['og_default_image.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'og_default_image_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('og_default_image.settings');

    $form['og_default_image_info'] = [
      '#markup' => $this->t("Upload the default OG image. Recommended dimensions are 1200x630px.<br>
                    You can then use the [og_default_image:og_default_image] token"),
    ];

    $form['og_default_image'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('OG Default Image'),
      '#upload_location' => 'public://og_default_images/',
      '#default_value' => $config->get('og_default_image') ?: [],
      '#upload_validators' => [
        'FileExtension' => ['extensions' => 'gif png jpg jpeg'],
        'FileSizeLimit' => ['fileLimit' => 5600000],
      ],
      '#description' => $this->t('Allowed extensions: png, jpg, jpeg.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    // Get the previously saved file ID if any.
    $config = $this->config('og_default_image.settings');
    $old_fid = $config->get('og_default_image') ? $config->get('og_default_image')[0] : FALSE;

    $this->config('og_default_image.settings')
      ->set('og_default_image', $form_state->getValue('og_default_image'))
      ->save();

    // Get the uploaded file ID.
    $fid = $form_state->getValue('og_default_image')[0] ?? NULL;
    if ($fid) {
      $this->setPermanent($fid);
    }

    // Delete old file.
    if ($old_fid &&  $old_fid !== $fid) {
      $this->deleteImage($old_fid);
    }

  }

  /**
   *
   */
  public function setPermanent($fid) {

    $file = $this->entityTypeManager->getStorage('file')->load($fid);
    // Check if the file was loaded successfully.
    if ($file) {
      // Flag the image as permanent.
      $file->setPermanent();
      $file->save();
    }
  }

  /**
   *
   */
  public function deleteImage($old_fid) {

    $old_file = $this->entityTypeManager->getStorage('file')->load($old_fid);
    if ($old_file) {
      // Delete the old file.
      $old_file->delete();
    }

  }

}
